"""
Class to create bytecode objects
"""
##########################################
# Imports
##########################################
import dis
import sys
import logging
import marshal
import py_compile
import types

def createBytecode(in_dir, in_file, ext):
    """
    @type in_file: string
    @param in_file: Name of module for which bytecode needs to be created

    @return: a list of instruction objects.
    @TODO: Generalize for modules with functions outside classes
    @TODO: Add error handling
    """
    log = logging.getLogger('Trace_Compare')
    out = []
    current_file_path = in_dir + "/" + in_file + ".py"
    try:
        pyc_file = in_dir + "/" + in_file + ".pyc"
        py_compile.compile(current_file_path, pyc_file)
        header_size = 16 if sys.version_info >= (3, 7) else 12
        with open(pyc_file, "rb") as f:
            magic_and_timestamp = f.read(header_size)  # first 8 or 12 bytes are metadata
            code = marshal.load(f)  # rest is a marshalled code object
    except:
        raise

    def inspect_code_object(co_obj):
        ins = dis.get_instructions(co_obj)
        out.extend(ins)
        for c in co_obj.co_consts:
            if isinstance(c, types.CodeType):
                inspect_code_object(c)

    inspect_code_object(code)
    return out



