import os
import sys
import graph_utils
######################################################################
# graph reachability using rules, after inc, high-level
######################################################################

# returns set of vertices reachable from vertices in s following edges in e.
# e is a set of edge, in the form of pairs of start and end vertices.
# s is a set of vertices.

# image(R,x) returns image of x under relation R, i.e. {y | (x,y) in R}
def image(pairSet,leftElem):
    return [p[1] for p in pairSet if p[0]==leftElem]

def infer(facts):
    W = facts
    r1reachu = []
    r1edgeuv = []
    R = []
    while len(W)!=0:
        x = W[0]
        if x[0]=='source':
            v = x[1][0]
            if ('reach', (v,)) not in R and ('reach', (v,)) not in W:
                W.append(('reach', (v,)))
        if x[0]=='reach':
            u = x[1][0]
            for lonely in image(r1edgeuv,[u]):
                v = lonely[0]
                if ('reach', (v,)) not in R and ('reach', (v,)) not in W:
                    W.append(('reach', (v,)))
            r1reachu.append([[u], []])

        if x[0]=='edge':
            u = x[1][0]
            v = x[1][1]
            for lonely in image(r1reachu,[u]):
                if ('reach', (v,)) not in R and ('reach', (v,)) not in W:
                    W.append(('reach', (v,)))
            r1edgeuv.append([[u], [v]])

        W.remove(x)
        R.append(x)
    return R

def reach_rule(e,s):
    facts = [('source', (x,)) for x in s]
    facts += [('edge', (x,y)) for (x,y) in e]
    result = infer(facts)
    return {x[0] for (type, x) in result if type == 'reach'}

def run_reach():
    s, e = graph_utils.read_graph_input(input_file)
    print('s =', s)
    print('e =',e)
    print('reach_rule =', reach_rule(e,s))

input_file = sys.argv[1] if len(sys.argv) > 1 else "graph.txt"
if os.path.exists(input_file):
    run_reach()
else:
    print('Please specify a valid input file.')
    exit(1)