import sys
import random

# Graph generation program that outputs the generated graph in graph.txt in the following format.
# each line describes one edge, represented by two natural numbers, interpreted as node names, separated by a space.
# 2 vertices are picked at random for graph reachability program input. This constitute the first line in the file.
# Program inputs: number N of nodes,number E of edges, with E >= N and an output file name.  Generate edges in two passes.
# pass 1: randomly generate either one in-edge or one out-edge for each node.
# pass 2: generate E-N additional edges, by selecting the source node and target node uniformly at random.
# If program is run directly with 2 input args N,E it generates the graph as described above else uses default values of 5 E, 5 N.


def generate_graph(N, E):
    graph = set()
    nodeList = range(N)
    #Pass 1
    for node in nodeList:
         p =random.random()
         graph.add((node, random.choice(nodeList))) if p > 0.5 else graph.add((random.choice(nodeList), node))
    #Pass 2
    while(len(graph) < E):
        edge = (random.choice(nodeList), random.choice(nodeList))
        graph.add(edge)
    return graph

def write_graph_input(s, graph, outfile):
    with open(outfile, "w") as out:
        out.write(str(s.pop()) + " " + str(s.pop()) + "\n")
        for edge in graph:
            out.write(str(edge[0]) + " " + str(edge[1]) + "\n")


def read_graph_input(input_file):
    s = set()
    e = set()
    fp = open(input_file, "r")
    for i, line in enumerate(fp):
        if i==0:
            nodes = line.rstrip('\n').split(" ")
            for n in nodes:
                s.add(int(n))
        else:
            nodes = line.rstrip('\n').split(' ')
            edge = tuple((int(nodes[0]), int(nodes[1])))
            e.add(edge)
    return s,e


if __name__ == '__main__':
    N = int(sys.argv[1]) if len(sys.argv) > 1 else 5
    E = int(sys.argv[2]) if len(sys.argv) > 2 else 5
    outfile = int(sys.argv[3]) if len(sys.argv) > 3 else "graph.txt"

    if E < N:
        print("Number of edges less than number of nodes.Cannot proceed.\n")
        exit(1)

    graph = generate_graph(N, E)
    print('Generated graph:', graph)

    s = set()
    while len(s) < 2: s.add(random.choice(range(N)))
    print('Input vertices:', s)

    write_graph_input(s, graph, outfile)
    print('Written to outfile:', outfile)