import sys

# Generic tree node class
class TreeNode(object):
    def __init__(self, val):
        self.val = val
        self.left = None
        self.right = None
        self.height = 1


# Recursive function to insert key in
# subtree rooted with node and returns
# new root of subtree.
def insert(root, key):
    # Step 1 - Perform normal BST
    if not root:
        return key
    elif key.val < root.val:
        root.left = insert(root.left, key)
    else:
        root.right = insert(root.right, key)

    # Step 2 - Update the height of the
    # ancestor node
    root.height = 1 + max(getHeight(root.left),
                          getHeight(root.right))

    # Step 3 - Get the balance factor
    balance = getBalance(root)

    # Step 4 - If the node is unbalanced,
    # then try out the 4 cases
    # Case 1 - Left Left
    if balance > 1 and key.val < root.left.val:
        return rightRotate(root)

    # Case 2 - Right Right
    if balance < -1 and key.val > root.right.val:
        return leftRotate(root)

    # Case 3 - Left Right
    if balance > 1 and key.val > root.left.val:
        root.left = leftRotate(root.left)
        return rightRotate(root)

    # Case 4 - Right Left
    if balance < -1 and key.val < root.right.val:
        root.right = rightRotate(root.right)
        return leftRotate(root)

    return root


def leftRotate(z):
    y = z.right
    T2 = y.left

    # Perform rotation
    y.left = z
    z.right = T2

    # Update heights
    z.height = 1 + max(getHeight(z.left),
                       getHeight(z.right))
    y.height = 1 + max(getHeight(y.left),
                       getHeight(y.right))

    # Return the new root
    return y


def rightRotate(z):
    y = z.left
    T3 = y.right

    # Perform rotation
    y.right = z
    z.left = T3

    # Update heights
    z.height = 1 + max(getHeight(z.left),
                       getHeight(z.right))
    y.height = 1 + max(getHeight(y.left),
                       getHeight(y.right))

    # Return the new root
    return y


def getHeight(root):
    if not root:
        return 0

    return root.height


def getBalance(root):
    if not root:
        return 0

    return getHeight(root.left) - getHeight(root.right)


def tree_search(x, k):
    """
        Recursively searches a binary tree for a particular key.
        Args:
            x: the node at which to start the search
        Returns:
            None, or the node containing the key.
    """
    if x is None:
        return None
    elif k == x.val:
        return x
    elif k < x.val:
        return tree_search(x.left, k)
    else:
        return tree_search(x.right, k)


def test_search():
    with open(sys.argv[1]) as file:
        file = file.read().split("\n")
        root = None
        for data in file:
            data = data.split("\t")
            # print(data, len(data))
            data[1] = int(data[1])
            if data[0] == "I":
                root = insert(root, TreeNode(data[1]))
            else:
                node = tree_search(root, data[1])
                if node:
                    print(node.val, "found")
                else:
                    print(data[1], "not found")


test_search()