import sys


# AVL TREE DEBUGGING

class Tree(object):
    def __init__(self, root=None):
        self.root = root


# Generic node class


class Node(object):
    def __init__(self, key):
        self.key = key
        self.left = None
        self.right = None
        self.height = 1
        self.p = None


# Iterative function to insert node in
# tree


def avl_insert(tree, z):
    # Step 1 - Perform normal BST
    w = tree.root
    y = None
    while (w != None):
        y = w
        if z.key > y.key:
            w = w.right
        else:
            w = w.left

    if y == None:
        tree.root = z
        y = tree.root
    elif z.key > y.key:
        y.right = z
        z.p = y
        if y.left == None:
            y.height += 1
    else:
        y.left = z
        z.p = y
        if y.right == None:
            y.height += 1

    # Step 2 - update heights and rotate if required
    # print("2")
    i = 0
    while (y != None):
        y.height = 1 + max(get_height(y.left),
                           get_height(y.right))
        balance = get_balance(y)
        if balance > 1 and z.key > y.left.key:
            left_rotate(tree, y.left)
            right_rotate(tree, y)

        # Case 2 - Right Right
        elif balance < -1 and z.key < y.right.key:
            right_rotate(tree, y.right)
            left_rotate(tree, y)

        # Case 3 - Left Right
        elif balance > 1 and z.key < y.left.key:
            right_rotate(tree, y)

        # Case 4 - Right Left
        elif balance < -1 and z.key > y.right.key:

            left_rotate(tree, y)

        y = y.p


def left_rotate(tree, x):
    assert (x.right != None)
    y = x.right
    x.right = y.left
    if y.left != None:
        y.left.p = x
    y.p = x.p
    if x.p == None:
        tree.root = y
    elif x == x.p.left:
        x.p.left = y
    else:
        x.p.right = y
    y.left = x
    x.p = y
    x.height = 1 + max(get_height(x.left),
                       get_height(x.right))
    y.height = 1 + max(get_height(y.left),
                       get_height(y.right))


def right_rotate(tree, x):
    assert (x.left != None)
    y = x.left
    x.left = y.right
    if y.right != None:
        y.right.p = x
    y.p = x.p
    if x.p == None:
        tree.root = y
    elif x == x.p.right:
        x.p.right = y
    else:
        x.p.left = y
    y.right = x
    x.p = y
    x.height = 1 + max(get_height(x.left),
                       get_height(x.right))
    y.height = 1 + max(get_height(y.left),
                       get_height(y.right))


def get_height(root):
    if not root:
        return 0

    return root.height


def get_balance(root):
    if not root:
        return 0

    return get_height(root.left) - get_height(root.right)


def iter_tree_search(x, k):
    # """
    #     Iteratively searches a binary tree for a particular key.
    #     Args:
    #         x: the node at which to start the search
    #     Returns:
    #         None, or the node containing the key.
    # """
    while x is not None and k != x.key:
        if k < x.key:
            x = x.left
        else:
            x = x.right
    return x


def test_search():
    with open(sys.argv[1]) as file:
        file = file.read().split("\n")
        tree = Tree()
        for data in file:
            data = data.split("\t")
            # print(data, len(data))
            data[1] = int(data[1])
            if data[0] == "I":
                avl_insert(tree, Node(data[1]))
            else:
                node = iter_tree_search(tree.root, data[1])
                if node:
                    print(node.key, "found")
                else:
                    print(data[1], "not found")


test_search()