import sys
import math
import result
from terminaltables import AsciiTable
import visualize
import collections


ROUND_DIGIT = 4


def diversity(dict_of_sets, type, algoList, outfile, original, visualizationfile):

    if not outfile == None:
        f = open(outfile, 'w')
        sys.stdout = f
        result_file = outfile + ".pickle"
    else:
        result_file = "results.pickle"

    result_obj = result.Results(result_file)
    ###
    # Printing number of hashes in each sets and
    # calculating the pairwise Jaccard
    ###
    print("--------------------------------------------")
    total = 0
    length = len(dict_of_sets)
    dict_of_sets = collections.OrderedDict(sorted(dict_of_sets.items()))
    table = [[x if y == 0 else y if x == 0 else 1 for x in range(
        length + 1)] for y in range(length + 1)]
    print("Hashes in each Fingerprints")
    list_of_sets = list(dict_of_sets.values())
    keys = list(dict_of_sets.keys())
    for i in range(length):
        print(i + 1, "-", keys[i], ":", len(list_of_sets[i]))
        for j in range(i + 1, length):
            jacc = jaccard(list_of_sets[i], list_of_sets[j])
            table[i + 1][j + 1] = jacc
            table[j + 1][i + 1] = jacc
            total = total + jacc

    ###
    # Printing pairwise Jaccard in the form of a table
    # and calculating standard deviation for the same
    ###
    if algoList[0]:
        print("--------------------------------------------\n\n")
        print("Pairwise Jaccard comparisons")
        table_ascii = AsciiTable(table)
        print(table_ascii.table)
        vtable = [table[x + 1][1:] for x in range(length)]

        avg = (total / (((length - 1) * (length)) / 2))
        dev = 0
        for i in range(length):
            for j in range(i + 1, length):
                dev = dev + ((table[i + 1][j + 1] - avg)
                             * (table[i + 1][j + 1] - avg))
        dev = dev / ((((length - 1) * (length)) / 2))
        stdev = round(math.sqrt(dev), ROUND_DIGIT)
        result_obj.add_Jaccard(table, avg, stdev)
        print("Average = ", round(avg, ROUND_DIGIT))
        print("Standard Deviation = ", stdev)
        print("--------------------------------------------\n\n")

    ###
    # Printing pairwise Smith in the form of a table
    # and calculating standard deviation for the same
    ###
    if algoList[1]:
        print("Pairwise Smith comparisons (similarity distance)")
        total = 0
        for i in range(length):
            for j in range(i + 1, length):
                s = smith(list_of_sets[i], list_of_sets[j])
                table[i + 1][j + 1] = s
                table[j + 1][i + 1] = s
                total = total + s
        table_ascii = AsciiTable(table)
        stable = [table[x + 1][1:] for x in range(length)]
        print(table_ascii.table)
        avg = (total / ((((length - 1) * (length)) / 2)))
        dev = 0
        for i in range(length):
            for j in range(i + 1, length):
                dev = dev + ((table[i + 1][j + 1] - avg)
                             * (table[i + 1][j + 1] - avg))
        dev = dev / ((((length - 1) * (length)) / 2))
        stdev = round(math.sqrt(dev), ROUND_DIGIT)
        result_obj.add_Smith(table, avg, stdev)
        print("Average = ", round(avg, ROUND_DIGIT))
        print("Standard Deviation = ", stdev)
        print("--------------------------------------------\n\n")

    ###
    # Printing group Jaccard and calculating standard deviation for the same
    ###
    if algoList[2]:
        print("Group comparisons: Jaccard")
        print("version A against the rest = |A intersect G\{A}| / |G|")
        print("--------------------------------------------")
        total = 0
        group_value = [0 for x in range(length)]
        for i in range(length):
            group = set()
            for j in range(length):
                if i != j:
                    group = group | list_of_sets[j]
            print(type, "#", (i + 1))
            jacc = jaccard(list_of_sets[i], group)
            group_value[i] = jacc
            total = total + jacc
            print(jacc)
            print("---------------------")
        avg = (total / length)
        dev = 0
        for i in range(length):
            dev = dev + ((group_value[i] - avg) * (group_value[i] - avg))
        dev = dev / length
        stdev = round(math.sqrt(dev), ROUND_DIGIT)
        result_obj.add_PJaccard(group_value, avg, stdev)
        print("Average = ", round(avg, ROUND_DIGIT))
        print("Standard Deviation = ", stdev)
        print("--------------------------------------------\n\n")

    ###
    # Printing shared fraction and calculating standard deviation for the same
    ###
    if algoList[3]:
        print("Group comparisons: shared fraction")
        print("fraction of A shared with group = |A intersect G\{A}| / |A|")
        print("--------------------------------------------")
        total = 0
        group_value1 = [0 for x in range(length)]
        for i in range(length):
            group = set()
            for j in range(length):
                if i != j:
                    group = group | list_of_sets[j]
            print(type, "#", (i + 1))
            jacc = jaccard2(list_of_sets[i], group)
            group_value1[i] = jacc
            total = total + jacc
            print(jacc)
            print("---------------------")
        avg = (total / length)
        dev = 0
        for i in range(length):
            dev = dev + ((group_value1[i] - avg) * (group_value1[i] - avg))
        dev = dev / length
        stdev = round(math.sqrt(dev), ROUND_DIGIT)
        result_obj.add_SharedFraction(group_value1, avg, stdev)
        print("Average = ", round(avg, ROUND_DIGIT))
        print("Standard Deviation = ", stdev)
        print("--------------------------------------------\n\n")

    #######
    print("Group comparisons: Set differences")
    print("version A minus the rest = |A setminus G\{A}| / |A|")
    print("--------------------------------------------")
    total = 0
    group_value1 = [0 for x in range(length)]
    for i in range(length):
        group = set()
        for j in range(length):
            if i != j:
                group = group | list_of_sets[j]
        print(type, "#", (i + 1))
        diff= len(list_of_sets[i].difference(group))
        setminus_res = round(diff/len(group), ROUND_DIGIT)
        group_value1[i] = setminus_res
        total = total + setminus_res
        print(str(diff) + "/"+ str(len(group))+ "=" + str(setminus_res))
        print("---------------------")
    avg = (total / length)
    dev = 0
    for i in range(length):
        dev = dev + ((group_value1[i] - avg) * (group_value1[i] - avg))
    dev = dev / length
    stdev = round(math.sqrt(dev), ROUND_DIGIT)
    result_obj.add_SetMinus(group_value1, avg, stdev)
    print("Average = ", round(avg, ROUND_DIGIT))
    print("Standard Deviation = ", stdev)
    print("--------------------------------------------\n\n")
    #####

    if visualizationfile is not None:
        visualize.createHeatMap(vtable, stable, keys,
                                group_value, group_value1, visualizationfile)
    ###
    # Printing Average and calculating standard deviation for the same
    ###
    if algoList[4]:
        masterList = []
        for i in range(length):
            masterList.extend(list(list_of_sets[i]))
        dict_count = [masterList.count(x) for x in masterList]

        print("Average number of sets in the group that each value appears in: ")
        avg = (sum(dict_count) / float(len(dict_count)))
        print(round((sum(dict_count) / float(len(dict_count))),
                    ROUND_DIGIT), "out of", length, "sets")
        dev = 0
        for value in dict_count:
            dev = dev + ((value - avg) * (value - avg))
        stdev = dev / len(dict_count)
        stdev = round(math.sqrt(stdev), ROUND_DIGIT)
        result_obj.add_Average(avg, stdev)
        print("Standard Deviation = ", round(stdev, ROUND_DIGIT))

        print("--------------------------------------------\n\n")
    result_obj.writeResult()


def jaccard(set1, set2):
    return round(len(set1 & set2) / len(set1 | set2), ROUND_DIGIT)


def jaccard2(set1, set2):
    return round((len(set1 & set2) / len(set1)), ROUND_DIGIT)


def smith(set1, set2):
    s1 = jaccard2(set1, set2)
    s2 = jaccard2(set2, set1)

    return round(math.sqrt(((s1 * s1) + (s2 * s2)) / 2), ROUND_DIGIT)


def setminus(set1, set2):
    return round(len(set1.difference(set2))/len(set2), ROUND_DIGIT)