import sys
import os
import graph_utils

######################################################################
# graph reachability using sets and loops, iterative, incrementalized
######################################################################

# returns set of vertices reachable from vertices in s following edges in e.
# e is a set of edge, in the form of pairs of start and end vertices.
# s is a set of vertices.

def reach_inc(e_set,s):
    # e is a map from each vertex with sucessors to its set of successors.
    e = {x:{y for (x2,y) in e_set if x2 == x} for (x,y) in e_set}
    w = set() #-- w = t - r = {} - {} = {}
    t = set() #-- t = e[r] = e[{}] = {}
    r = set()
    for x in s:
        if x in e:             #|| same as the body
            for y in e[x]:       #|| of the while loop,
                if y not in t:     #|| after while exists x in w
                    if y not in r: #||
                        w.add(y)   #||
                    t.add(y)       #||
        if x in t:             #||
            w.remove(x)          #||
        r.add(x)               #||
    while w:              # while exists x in w:
        x = next(iter(w)) #
        if x in e:             # for y in e{x} #|| maintain t
            for y in e[x]:       #               #|| under r add x
                if y not in t:                     #||
                    if y not in r:                         #|| maintain w
                        w.add(y)                           #|| under t add y
                    t.add(y)                       #||
        if x in t:                                     #|| maintain w
            w.remove(x)                                #|| under r add x
        r.add(x)
    return r

def run_reach():
    s, e = graph_utils.read_graph_input(input_file)
    print('s =', s)
    print('e =',e)
    print('reach_inc =', reach_inc(e,s))

print(sys.argv[1])
input_file = sys.argv[1] if len(sys.argv) > 1 else "graph.txt"
if os.path.exists(input_file):
    run_reach()
else:
    print('Please specify a valid input file.')
    exit(1)
