class ngram(object):
    # List of constants
    def __init__(self):
        self.relative_constants = {110, 111, 112, 113, 120, 121, 122}
        self.variable_index = {90, 91, 95, 96, 97, 98, 101, 105,
                          107, 108, 116, 124, 125, 126, 135, 136, 137}
        self.ngram_n = 5
        self.window_size = 10
        self.rabins_constant = 4657
        self.dict_of_sets = {}


    def getFingerprint(self, trace_bytecode_map, window_size_in, ngram_n_in, do_winnow, original):
        self.ngram_n = ngram_n_in
        self.window_size = window_size_in

        for file, bytecode_sequence in trace_bytecode_map.items():
            self.file_walk(file, bytecode_sequence, do_winnow, False)
        if not original is None:
            self.file_walk(original, bytecode_sequence, do_winnow, True)
        return self.dict_of_sets


    def file_walk(self, currentFile, instructions, do_winnow, original):
        inst_hash = []

        for i in range(len(instructions) - (self.ngram_n - 1)):
            inst_hash.append(self.getRabinHash(instructions[i:i + self.ngram_n]))
        if inst_hash:
            if original:
                key = "!!Original - " + currentFile
            else:
                key = currentFile
            if do_winnow == "False":
                self.dict_of_sets[key] = set(inst_hash)
            else:
                self.dict_of_sets[key] = set(self.winnow(inst_hash))


    def winnow(self, inst_hash):
        h = []
        fingerprint = []
        hash_iter = iter(inst_hash)
        for i in range(self.window_size):
            tmp = next(hash_iter, None)
            if tmp != None:
                h.append(tmp)
        r = self.window_size - 1
        min = self.window_size - 1

        while len(h) > min and True:
            if min == r:
                i = (r - 1 + self.window_size) % self.window_size
                while i != r:
                    if h[i] < h[min]:
                        min = i
                    i = (i - 1 + self.window_size) % self.window_size
                fingerprint.append(h[min])
            else:
                if h[r] <= h[min]:
                    min = r
                    fingerprint.append(h[min])
            r = (r + 1) % self.window_size
            h[r] = next(hash_iter, None)
            if h[r] == None:
                break
        return fingerprint

    def getHash(self, n_instructions):
        variable_list = []
        # Variables are stored in a list, while making sure there are no duplicates.
        # The index from this list is then used instead of the variable.
        # For example, variables appear in the following order in the bytecode: 3,2,6,2,9
        # They are stored in a list as [3,2,6,9] and while calculating the hash value index from this list
        # will be used instead of the variable. Thus values used would be 0 for 3,
        # 1 for 2, 2 for 6 and 3 for 9
        for instruction in n_instructions:
            if instruction.opcode in self.variable_index and instruction.arg not in variable_list:
                variable_list.append(instruction.arg)

        hash = 0
        for instruction in n_instructions:
            hash = hash << 8
            if instruction.opcode < 90 or instruction.opcode == 99 or instruction.opcode == 104:
                # If the instruction has no operands
                hash += instruction.opcode
            elif instruction.opcode in self.relative_constants:
                # If the operand is a constant value with relative line number
                hash += instruction.opcode
            elif instruction.opcode in self.variable_index:
                # If the operand is a variable.
                hash += 256 * (variable_list.index(instruction.arg)
                               ) + instruction.opcode
            else:
                #
                hash += 256 * (instruction.arg) + instruction.opcode
        return hash


    def getRabinHash(self, n_instructions):
        variable_list = []
        # Variables are stored in a list, while making sure there are no duplicates.
        # The index from this list is then used instead of the variable.
        # For example, variables appear in the following order in the bytecode: 3,2,6,2,9
        # They are stored in a list as [3,2,6,9] and while calculating the hash value index from this list
        # will be used instead of the variable. Thus values used would be 0 for 3,
        # 1 for 2, 2 for 6 and 3 for 9
        for instruction in n_instructions:
            if instruction.opcode in self.variable_index and instruction.arg not in variable_list:
                variable_list.append(instruction.arg)

        hash = 0
        for instruction in n_instructions:
            hash = hash * self.rabins_constant
            if instruction.opcode < 90 or instruction.opcode == 99 or instruction.opcode == 104:
                # If the instruction has no operands
                hash += instruction.opcode
            elif instruction.opcode in self.relative_constants:
                # If the operand is a constant value with relative line number
                hash += instruction.opcode
            elif instruction.opcode in self.variable_index:
                # If the operand is a variable.
                hash += 256 * (variable_list.index(instruction.arg)
                               ) + instruction.opcode
            else:
                #
                hash += 256 * (instruction.arg) + instruction.opcode
        return hash
