import argparse
import configparser
import os
import sys
import re

def filter_trace(file_blacklist, function_blacklist):
    function_blacklistFlag= False
    for line in sys.stdin:

        # If functionblacklistflag is True, skip all lines till the next occurrence of --modulename (new function)
        if "--- modulename" in line and function_blacklistFlag == True:
            function_blacklistFlag = False

        if function_blacklistFlag:
            continue

        for item in function_blacklist:
            module_name, func_name = item.split(".")
            module_name = module_name.strip()
            func_name = func_name.strip()
            if "--- modulename: "+ module_name in line and "funcname: "+func_name in line:
                function_blacklistFlag = True
                break

        if function_blacklistFlag:
            continue

        blacklist_flag= False
        line = str(line.replace('\n', ''))
        line = line.rstrip(":")
        line = line.split(":")
        # Trace generated by the trace module is always of the form <file_name>(<line_num>): <line of code>
        # This checks below ensure the above

        check = line[0]
        # Skip empty lines in the trace
        if len(line[0]) == 0:
            continue

        if not bool(re.match(".*\(.*\)", check)):
            continue
        #Trace module adds 1 line for the the ending of multiline comment immediately after class declaration
        #This condition helps skip the comment trace
        if "\"\"\"" in line[1]:
            continue
        if len(file_blacklist) == 0:
            print(check)
        else:
            for item in file_blacklist:
                item = item.strip()
                if item in check:
                    blacklist_flag = True
            if not blacklist_flag:
                print(check)

if __name__ == "__main__":
    parser = argparse.ArgumentParser("trace filter")
    parser.add_argument('-c', action='store',
                        dest='select_file',
                        help='Read folder location from INI file')

    file_blacklist = []
    function_blacklist=[]
    # Result of command line arguments
    results = parser.parse_args()

    if results.select_file is not None:
        setting_file = results.select_file
        config = configparser.ConfigParser()

        if not os.path.isfile(setting_file):
            print('Please specify a valid configuration file')
            exit(1)

        config.read(setting_file)
        if "Settings_Filter" in config:
            settings = config['Settings_Filter']
            if "File_Blacklist" in settings:
                file_blacklist.extend(settings.get('File_Blacklist', []).split(','))
            if "Function_Blacklist" in settings:
                function_blacklist.extend(settings.get('Function_Blacklist', []).split(','))
            src = settings.get('Source', os.path.dirname(setting_file))
        else:
            print("Selected config file does not contain relevant options")
            exit(1)

    filter_trace(file_blacklist, function_blacklist)