import Levenshtein
import sys
import time
from terminaltables import AsciiTable
import plotly as py
import plotly.graph_objs as go
from collections import OrderedDict
import os

def createHeatMap(data, data2, data3, list_of_files, vfile):

    data1 = go.Heatmap(
        z=data,
        x=list_of_files,
        y=list_of_files,
        showscale=False
    )

    data2 = go.Heatmap(
        z=data2,
        x=list_of_files,
        y=list_of_files,
        showscale=False
    )
    data3 = go.Heatmap(
        z=data3,
        x=list_of_files,
        y=list_of_files,
        showlegend=False
    )

    data = [data1, data2, data3]

    fig = py.tools.make_subplots(rows=2, cols=2, subplot_titles=(
        'edit distance',
        'edit distance  / average(len1,len2)',
        '(edit distance - abs(len2-len1)) / min(len1,len2)'),
        print_grid=False, horizontal_spacing=0.20)
    fig.append_trace(data1, 1, 1)
    fig.append_trace(data2, 1, 2)
    fig.append_trace(data3, 2, 1)

    fig['layout'].update(height=900, width=1200,
                         title='Pairwise comparison of Edit_Distance',
                         margin=dict(t=100, b=100, l=200, r=50))
    py.offline.plot(fig, filename=vfile)


def editDistance(bytecode_seq1, bytecode_seq2):
    def seqLog(instr, s):
        try:
            if instr.arg == None:
                s += chr(int(instr.opcode))
            else:
                s += chr(int(instr.opcode)) + chr(int(instr.arg))
        except:
                print('An error occurred in converting opcode and arg to 16bit value for edit distance.Integer overflow.')
                exit(1)
    start_time = time.time()
    s1 = []
    for instr in bytecode_seq1:
        seqLog(instr, s1)
    s2 = []
    for instr in bytecode_seq2:
        seqLog(instr, s2)
    dist = Levenshtein.distance("".join(s1), "".join(s2))
    print("--- %s seconds ---" % (time.time() - start_time),
          "taken for edit distance between")
    return (dist, len(s1), len(s2))


def editDistanceSeq(trace_bytecode_map, vfile, outfile):

    files = list(trace_bytecode_map.keys())
    length = len(files)
    table1 = [[x if y == 0 else y if x == 0 else 0 for x in range(
        length + 1)] for y in range(length + 1)]
    table2 = [[x if y == 0 else y if x == 0 else 0 for x in range(
        length + 1)] for y in range(length + 1)]
    table3 = [[x if y == 0 else y if x == 0 else 0 for x in range(
        length + 1)] for y in range(length + 1)]
    list_of_files = OrderedDict()

    for i in range(length):
        for j in range(i + 1, length):

            res = editDistance(trace_bytecode_map[files[i]], trace_bytecode_map[files[j]])
            print(files[i], files[j])
            table1[i + 1][j + 1] = res[0]
            table1[j + 1][i + 1] = table1[i + 1][j + 1]

            table2[i + 1][j + 1] = res[0] / ((res[1] + res[2]) / 2)
            table2[j + 1][i + 1] = table2[i + 1][j + 1]

            table3[i + 1][j + 1] = (res[0] -
                                    abs(res[1] - res[2])) / min(res[1], res[2])
            table3[j + 1][i + 1] = table3[i + 1][j + 1]

            list_of_files[str(i + 1) + "." + files[i]] = res[1]
            list_of_files[str(j + 1) + "." + files[j]] = res[2]

    # saving stdout to a outfile
    f = open(outfile, 'a')
    sys.stdout = f

    print("length of files")
    for key, value in list_of_files.items():
        print(key.split(".", 1)[0] + ". " +
              key.split("\\")[-1] + " - " + str(value))

    print("\nEdit_Distance")
    table_ascii = AsciiTable(table1)
    print(table_ascii.table)
    print("\nEdit_Distance / avg length")
    table_ascii = AsciiTable(table2)
    print(table_ascii.table)
    print("\nEdit_Distance - diff length / min length")
    table_ascii = AsciiTable(table3)
    print(table_ascii.table)

    table1 = [row[1:] for row in table1[1:]]
    table2 = [row[1:] for row in table2[1:]]
    table3 = [row[1:] for row in table3[1:]]
    list_of_files = [os.path.basename(file)
                     for file in list(list_of_files.keys())]

    createHeatMap(table1, table2, table3, list_of_files, vfile)